@@ -1,42 +1,37 @@
-#' R function for calculating accumulated cost of movement across the terrain and least-cost paths from an origin
+#' R function for calculating accumulated anisotropic cost of movement across the terrain and least-cost paths from an origin
 #'
-#' The function provides the facility to calculate the accumulated cost of movement around a starting location and to optionally calculate least-cost path(s) toward
+#' The function provides the facility to calculate the anisotropic accumulated cost of movement around a starting location and to optionally calculate least-cost path(s) toward
 #' one or multiple destinations. It implements different cost estimations related to human movement across the landscape.
 #' The function takes as input a Digital Terrain Model ('RasterLayer' class) and a point feature ('SpatialPointsDataFrame' class), the latter representing
 #' the starting location, i.e. the location from which the accumulated cost is calculated. \cr
 #'
-#' If the parameter 'destin' is fed with a dataset representing destination location(s) ('SpatialPointsDataFrame' class), the function will also calculate
+#' If the parameter 'destin' is fed with a dataset representing destination location(s) ('SpatialPointsDataFrame' class), the function also calculate
 #' least-cost path(s) plotted on the input DTM; the length of each path will be saved under the variable 'length' stored in the 'LCPs' dataset ('SpatialLines' class) returned by the function.
 #' The red dot(s) representing the destination location(s) will be labelled with numeric values representing
 #' the cost value at the location(s). The cost value will be also appended to the updated destination dataset returned by the function and
 #' storing a new variable named 'cost'.\cr
 #'
-#' The function builds on functions out of Jacob van Etten's 'gdistance' package, and by default uses a 16-directions movement in calculating the accumulated cost-surface.
-#' The number of movements can be optionally set by the user via the 'moves' parameter.
+#' The function builds on functions out of Jacob van Etten's 'gdistance' package.
+#' Under the hood, movecost() calculates the slope as rise over run, following the procedure described
+#' by van Etten, "R Package gdistance: Distances and Routes on Geographical Grids" in Journal of Statistical Software 76(13), 2017, pp. 14-15.\cr
 #'
-#' The slope is internally calculated by the function using the 'terrain()' function out of the 'raster' package, using 8 neighbors.
-#' The slope is calculated in degrees and, for the sake of its use within the implemented cost functions, it is internally modified (i.e., turned into either gradient or percent)
-#' according to the user-selected cost function. The user can input a slope dataset (RasterLayer class) using the parameter 'slope'; this can prove usefull
-#' in cases when the slope has to be preliminarily modified, for instance to mask out areas that cannot be crossed or to weight the slope values according
-#' to a user-defined weighting scheme.\cr
-#'
-#' The following cost functions are implemented (\strong{x} stands for \strong{slope}):\cr
+#' The following cost functions are implemented (\strong{x[adj]} stands for slope as rise/run calculated for adjacent cells):\cr
 #'
 #' \strong{Tobler's hiking function (on-path) (speed in kmh)}:\cr
 #'
-#' \eqn{ 6 * exp(-3.5 * abs(tan(x*pi/180) + 0.05)) }\cr
+#' \eqn{ 6 * exp(-3.5 * abs(x[adj] + 0.05)) }\cr
 #'
 #'
 #' \strong{Tobler's hiking function (off-path) (speed in kmh)}:\cr
 #'
-#' \eqn{ (6 * exp(-3.5 * abs(tan(x*pi/180) + 0.05))) * 0.6 }\cr
+#' \eqn{ (6 * exp(-3.5 * abs(x[adj] + 0.05))) * 0.6 }\cr
 #'
 #' as per Tobler's indication, the off-path walking speed is reduced by 0.6.\cr
 #'
 #'
 #' \strong{Marquez-Perez et al.'s modified Tobler hiking function (speed in kmh)}:\cr
 #'
-#' \eqn{ 4.8 * exp(-5.3 * abs((tan(x*pi/180) * 0.7) + 0.03)) }\cr
+#' \eqn{ 4.8 * exp(-5.3 * abs((x[adj] * 0.7) + 0.03)) }\cr
 #'
 #' modified version of the Tobler's hiking function as proposed by Joaquin Marquez-Parez, Ismael Vallejo-Villalta & Jose I. Alvarez-Francoso (2017), "Estimated travel time for walking trails in natural areas",
 #' Geografisk Tidsskrift-Danish Journal of Geography, 117:1, 53-62, DOI: 10.1080/00167223.2017.1316212.\cr
@@ -44,34 +39,33 @@
 #'
 #' \strong{Irmischer-Clarke's modified Tobler hiking function (on-path)}:\cr
 #'
-#' \eqn{ (0.11 + exp(-(tan(x*pi/180)*100 + 5)^2 / (2 * 30)^2)) * 3.6 }\cr
+#' \eqn{ (0.11 + exp(-(abs(x[adj])*100 + 5)^2 / (2 * 30)^2)) * 3.6 }\cr
 #'
 #' modified version of the Tobler's function as proposed for (male) on-path hiking by Irmischer, I. J., & Clarke, K. C. (2018). Measuring and modeling the speed of human navigation.
 #' Cartography and Geographic Information Science, 45(2), 177-186. https://doi.org/10.1080/15230406.2017.1292150. \strong{Note}: the function originally expresses speed in m/s; it has been is reshaped (multiplied by 3.6)
-#' to turn it into kmh for consistency with the other Tobler-related cost functions; also, originally the slope is in percent; \eqn{tan(x*pi/180)*100} turns the slope from degrees to percent (=rise/run*100).\cr
+#' to turn it into kmh for consistency with the other Tobler-related cost functions.\cr
 #'
 #'
 #'\strong{Irmischer-Clarke's modified Tobler hiking function (off-path)}:\cr
 #'
-#' \eqn{ (0.11 + 0.67 * exp(-(tan(x*pi/180)*100 + 2)^2 / (2 * 30)^2)) * 3.6 }\cr
+#' \eqn{ (0.11 + 0.67 * exp(-(abs(x[adj])*100 + 2)^2 / (2 * 30)^2)) * 3.6 }\cr
 #'
 #'
 #'\strong{Uriarte Gonzalez's slope-dependant walking-time cost function}:\cr
 #'
-#' \eqn{ 1/ (0.0277 * (tan(x*pi/180)*100) + 0.6115) }\cr
+#' \eqn{ 1/ (0.0277 * (abs(x[adj])*100) + 0.6115) }\cr
 #'
 #' proposed by Uriarte Gonzalez;
 #' \strong{see}: Chapa Brunet, T., Garcia, J., Mayoral Herrera, V., & Uriarte Gonzalez, A. (2008). GIS landscape models for the study of preindustrial settlement patterns in Mediterranean areas.
 #' In Geoinformation Technologies for Geo-Cultural Landscapes (pp. 255-273). CRC Press. https://doi.org/10.1201/9780203881613.ch12.\cr
 #' The cost function is originally expressed in seconds; for the purpose of its implementation in this function, it is the reciprocal of time (1/T) that is used in order to eventually get
-#' T/1. Also, originally the slope is in percent: \eqn{tan(x*pi/180)*100} turns the slope from degrees to percent (=rise/run*100).
-#' Unlike the original cost function, here the pixel resolution is not taken into account since 'gdistance' takes care of the cells' dimension
+#' T/1. Unlike the original cost function, here the pixel resolution is not taken into account since 'gdistance' takes care of the cells' dimension
 #' when calculating accumulated costs.
 #'
 #'
-#' \strong{Relative energetic expenditure cost function}:\cr
+#'\strong{Relative energetic expenditure cost function}:\cr
 #'
-#' \eqn{ 1 / (tan(x*pi/180) / tan (1*pi/180)) }\cr
+#' \eqn{ 1 / (tan((atan(abs(x[adj]))*180/pi)*pi/180) / tan (1*pi/180)) }\cr
 #'
 #' slope-based cost function expressing change in potential energy expenditure;
 #' \strong{see} Conolly, J., & Lake, M. (2006). Geographic Information Systems in Archaeology. Cambridge: Cambridge University Press, p. 220;
@@ -82,14 +76,14 @@
 #'
 #' \strong{Herzog's metabolic cost function in J/(kg*m)}:\cr
 #'
-#' \eqn{ 1 / ((1337.8 * tan(x*pi/180)^6 + 278.19 * tan(x*pi/180)^5 - 517.39 * tan(x*pi/180)^4 - 78.199 * tan(x*pi/180)^3 + 93.419 * tan(x*pi/180)^2 + 19.825 * tan(x*pi/180) + 1.64)) }\cr
+#' \eqn{ 1 / ((1337.8 * abs(x[adj])^6) + (278.19 * abs(x[adj])^5) - (517.39 * abs(x[adj])^4) - (78.199 * abs(x[adj])^3) + (93.419 * abs(x[adj])^2) + (19.825 * abs(x[adj])) + 1.64) }\cr
 #'
 #' \strong{see} Herzog, I. (2016). Potential and Limits of Optimal Path Analysis. In A. Bevan & M. Lake (Eds.), Computational Approaches to Archaeological Spaces (pp. 179-211). New York: Routledge.\cr
 #'
 #'
 #' \strong{Wheeled-vehicle critical slope cost function}:\cr
 #'
-#' \eqn{ 1 / (1 + ((tan(x*pi/180)*100) / sl.crit)^2)  }\cr
+#' \eqn{ 1 / (1 + ((abs(x[adj])*100) / sl.crit)^2)  }\cr
 #'
 #' where \eqn{sl.crit} (=critical slope, in percent) is "the transition where switchbacks become more effective than direct uphill or downhill paths" and typically is in the range 8-16;
 #' \strong{see} Herzog, I. (2016). Potential and Limits of Optimal Path Analysis. In A. Bevan & M. Lake (Eds.), Computational Approaches to Archaeological Spaces (pp. 179-211). New York: Routledge. \cr
@@ -97,7 +91,7 @@
 #'
 #' \strong{Pandolf et al.'s metabolic energy expenditure cost function (in Watts)}:\cr
 #'
-#' \eqn{ 1 / (1.5 * W + 2.0 * (W + L) * (L / W)^2 + N * (W + L) * (1.5 * V^2 + 0.35 * V * (tan(x*pi/180)*100))) }\cr
+#' \eqn{ 1 / (1.5 * W + 2.0 * (W + L) * (L / W)^2 + N * (W + L) * (1.5 * V^2 + 0.35 * V * (abs(x[adj])*100))) }\cr
 #'
 #' where \eqn{W} is the walker's body weight (Kg), \eqn{L} is the carried load (in Kg), \eqn{V} is the velocity in m/s, \eqn{N} is a coefficient representing ease of movement on the terrain.\cr
 #'
@@ -114,7 +108,7 @@
 #'
 #' \strong{Van Leusen's metabolic energy expenditure cost function (in Watts)}:\cr
 #'
-#' \eqn{ 1 / (1.5 * W + 2.0 * (W + L) * (L / W)^2 + N * (W + L) * (1.5 * V^2 + 0.35 * V * (tan(x*pi/180)*100) + 10))  }\cr
+#' \eqn{ 1 / (1.5 * W + 2.0 * (W + L) * (L / W)^2 + N * (W + L) * (1.5 * V^2 + 0.35 * V * abs(x[adj])*100) + 10))  }\cr
 #'
 #' which modifies the Pandolf et al.'s equation; \strong{see} Van Leusen, P. M. (2002). Pattern to process: methodological investigations into the formation and interpretation of spatial patterns in archaeological landscapes. University of Groningen.\cr
 #' \strong{Note} that, as per Herzog, I. (2013). Least-cost Paths - Some Methodological Issues, Internet Archaeology 36 (http://intarch.ac.uk/journal/issue36/index.html) and
@@ -131,14 +125,13 @@
 #' When using the Tobler-related cost functions, the time unit can be selected by the user setting the 'time' parameter to 'h' (hour) or to 'm' (minutes).\cr
 #'
 #' In general, the user can also select which type of visualization the function has to produce; this is achieved setting the 'outp' parameter to either 'r' (=raster)
-#' or to 'c' (=contours). The former will produce a raster image with a colour scale and contour lines representing the accumulated cost surface; the latter parameter will only
+#' or to 'c' (=contours). The former will produce a raster with a colour scale and contour lines representing the accumulated cost surface; the latter parameter will only
 #' produce contour lines.\cr
 #'
 #' The contour lines' interval is set using the parameter 'breaks'; if no value is passed to the parameter, the interval will be set by default to
 #' 1/10 of the range of values of the accumulated cost surface.\cr
 #'
 #' @param dtm digital terrain model (RasterLayer class).
-#' @param slope slope dataset (RasterLayer class); if NULL (default), the slope (in degree) is internally calculated (see Details).
 #' @param origin location from which the walking time is computed (SpatialPointsDataFrame class).
 #' @param destin location(s) to which least-cost path(s) is calculated (SpatialPointsDataFrame class).
 #' @param funct cost function to be used: \strong{t} (default) uses the on-path Tobler's hiking function;
@@ -155,11 +148,10 @@
 #' 'h' for hour, 'm' for minutes.
 #' @param outp type of output: 'raster' or 'contours' (see Details).
 #' @param sl.crit critical slope (in percent), typically in the range 8-16 (10 by default) (used by the wheeled-vehicle cost function; see Details).
-#' @param W walker's body weight (in Kg; used by the Pandolf's and Van Leusen's cost function; see Details).
-#' @param L carried load weight (in Kg; used by the Pandolf's and Van Leusen's cost function; see Details).
+#' @param W walker's body weight (in Kg; 70 by default; used by the Pandolf's and Van Leusen's cost function; see Details).
+#' @param L carried load weight (in Kg; 0 by default; used by the Pandolf's and Van Leusen's cost function; see Details).
 #' @param N coefficient representing ease of movement (1 by default) (used by the Pandolf's and Van Leusen's cost function; see Details).
 #' @param V speed in m/s (1.2 by default) (used by the Pandolf's and Van Leusen's cost function; see Details).
-#' @param moves number of directions used when computing the accumulated cost-surface (16 by default).
 #' @param breaks isolines interval; if no value is supplied, the interval is set by default to 1/10 of the range of values of the accumulated cost surface.
 #' @param cont.lab if set to TRUE (default) display the labels of the contours over the accumulated cost surface.
 #' @param destin.lab if set to TRUE (default) display the label(s) indicating the cost at the destination location(s).
@@ -178,7 +170,8 @@
 ##' }
 #' @keywords movecost
 #' @export
-#' @importFrom grDevices terrain.colors
+#' @importFrom raster ncell mask
+#' @importFrom grDevices terrain.colors topo.colors
 #' @importFrom graphics layout par
 #' @examples
 #' # load a sample Digital Terrain Model
@@ -197,19 +190,24 @@
 #' # and plotted
 #' result <- movecost(dtm=volc,origin=volc.loc, destin=destin.loc, breaks=0.05)
 #'
-movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h", outp="r", sl.crit=10, W=70, L=0, N=1, V=1.2, moves=16, breaks=NULL, cont.lab=TRUE, destin.lab=TRUE, cex.breaks=0.6, cex.lcp.lab=0.6, oneplot=TRUE, export=FALSE){
+movecost <- function (dtm, origin, destin=NULL, funct="t", time="h", outp="r", sl.crit=10, W=70, L=0, N=1, V=1.2, breaks=NULL, cont.lab=TRUE, destin.lab=TRUE, cex.breaks=0.6, cex.lcp.lab=0.6, oneplot=TRUE, export=FALSE){
 
-  #calculate the terrain slope in degrees
-  if(is.null(slope)==TRUE){
-    slope <- raster::terrain(dtm, opt='slope', unit='degrees', neighbors=8)
-  } else {
-    slope <- slope
-  }
+  #deactivate the warning messages because a warning that can be safely ignored will be produced by the procedure
+  #used to get slope as rise over run
+  options(warn = -1)
+
+  #calculate the altitudinal difference between adjacent cells
+  altDiff <- function(x){x[2] - x[1]}
+  hd <- gdistance::transition(dtm, altDiff, 8, symm=FALSE)
+
+  #use the geoCorrection function to divide the altitudinal difference by the distance between cells
+  #so getting slope as rise over run
+  slope <- gdistance::geoCorrection(hd)
 
   #define different types of cost functions and set the appropriate text to be used for subsequent plotting
   if (funct=="t") {
-    #Tobler's hiking function; kmh; tan(x*pi/180) turns the slope from degrees to rise over run
-    cost_function <- function(x){6 * exp(-3.5 * abs(tan(x*pi/180) + 0.05))}
+    #Tobler's hiking function; kmh
+    cost_function <- function(x){6 * exp(-3.5 * abs(x[adj] + 0.05))}
 
     #set the labels to be used within the returned plot
     main.title <- paste0("Walking-time isochrones (in ", time, ") around origin")
@@ -219,9 +217,9 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
   }
 
   if (funct=="tofp") {
-    #Tobler's hiking function off-path routes; kmh; tan(x*pi/180) turns the slope from degrees to rise over run
+    #Tobler's hiking function off-path routes; kmh
     #note that the multiplier 0.6 suggested by Tobler is meant to reduce the off-path walking speed
-    cost_function <- function(x){(6 * exp(-3.5 * abs(tan(x*pi/180) + 0.05))) * 0.6}
+    cost_function <- function(x){(6 * exp(-3.5 * abs(x[adj] + 0.05))) * 0.6}
 
     #set the labels to be used within the returned plot
     main.title <- paste0("Walking-time isochrones (in ", time, ") around origin")
@@ -231,8 +229,8 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
   }
 
   if(funct=="mt") {
-    #Marquez-Perez et al.'s modified Tobler hiking function; kmh; tan(x*pi/180) turns the slope from degrees to rise over run
-    cost_function <- function(x){4.8 * exp(-5.3 * abs((tan(x*pi/180) * 0.7) + 0.03))}
+    #Marquez-Perez et al.'s modified Tobler hiking function; kmh
+    cost_function <- function(x){4.8 * exp(-5.3 * abs((x[adj] * 0.7) + 0.03))}
 
     #set the labels to be used within the returned plot
     main.title <- paste0("Walking-time isochrones (in ", time, ") around origin")
@@ -244,8 +242,8 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
   if(funct=="ic") {
     #Irmischer-Clarke's modified Tobler hiking function; originally in m/s (males, on-path);
     # the formula is reshaped (multiplied by 3.6) below to turn it into kmh for consistency with the other Tobler-related cost functions;
-    # also, originally the slope is in percent; tan(x*pi/180)*100 turns the slope from degrees to percent (=rise/run*100)
-    cost_function <- function(x){(0.11 + exp(-(tan(x*pi/180)*100 + 5)^2 / (2 * 30)^2)) * 3.6}
+    # Slope in percent.
+    cost_function <- function(x){(0.11 + exp(-(abs(x[adj])*100 + 5)^2 / (2 * 30^2))) * 3.6}
 
     #set the labels to be used within the returned plot
     main.title <- paste0("Walking-time isochrones (in ", time, ") around origin")
@@ -257,8 +255,8 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
   if(funct=="icofp") {
     #Irmischer-Clarke's modified Tobler hiking function; originally in m/s (males, off-path);
     # the formula is reshaped (multiplied by 3.6) below to turn it into kmh for consistency with the other Tobler-related cost functions;
-    # also, originally the slope is in percent; tan(x*pi/180)*100 turns the slope from degrees to percent (=rise/run*100)
-    cost_function <- function(x){(0.11 + 0.67 * exp(-(tan(x*pi/180)*100 + 2)^2 / (2 * 30)^2)) * 3.6}
+    # Slope in percent.
+    cost_function <- function(x){(0.11 + 0.67 * exp(-(abs(x[adj])*100 + 2)^2 / (2 * 30^2))) * 3.6}
 
     #set the labels to be used within the returned plot
     main.title <- paste0("Walking-time isochrones (in ", time, ") around origin")
@@ -270,10 +268,10 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
   if(funct=="ug") {
     #Antonio Uriarte Gonzalez's slope-dependant walking-time cost function;
     # the cost function is originally expressed in seconds; here it is the reciprocal of time (1/T) that is used in order to eventually get
-    #T/1. Also, originally the slope is in percent; tan(x*pi/180)*100 turns the slope from degrees to percent (=rise/run*100).
+    #T/1. Slope is in percent.
     #Note: unlike the original formula, here the pixel resolution is not taken into account since 'gdistance' takes care of the cells' dimension
     #when calculating accumulated costs.
-    cost_function <- function(x){ 1/ (0.0277 * (tan(x*pi/180)*100) + 0.6115) }
+    cost_function <- function(x){ 1/ (0.0277 * (abs(x[adj])*100) + 0.6115) }
 
     #set the labels to be used within the returned plot
     main.title <- paste0("Walking-time isochrones (in ", time, ") around origin")
@@ -284,8 +282,9 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
 
   if(funct=="ree") {
     #relative energetic expenditure;
-    # to calculate tangent of degrees (as requested by the cost function) we must first convert degrees to radians by multypling by pi/180
-    cost_function <- function(x){ 1 / (tan(x*pi/180) / tan (1*pi/180)) }
+    # to calculate tangent of degrees (as requested by the cost function) we must first convert degrees to radians by multypling by pi/180;
+    #(atan(abs(x[adj]))*180/pi) turns rise/run into degrees, which are then converted into radians before calculating the tangent
+    cost_function <- function(x){ 1 / (tan((atan(abs(x[adj]))*180/pi)*pi/180) / tan (1*pi/180)) }
 
     #set the labels to be used within the returned plot
     main.title <- "Accumulated cost isolines around origin"
@@ -296,8 +295,8 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
 
   if(funct=="hrz") {
     #Herzog metabolic cost function in J/(kg*m);
-    #tan(x*pi/180) turns slope from degrees to rise/run, which is requested by the cost function
-    cost_function <- function(x){ 1 / ((1337.8 * tan(x*pi/180)^6 + 278.19 * tan(x*pi/180)^5 - 517.39 * tan(x*pi/180)^4 - 78.199 * tan(x*pi/180)^3 + 93.419 * tan(x*pi/180)^2 + 19.825 * tan(x*pi/180) + 1.64)) }
+    #rise/run is requested by the cost function
+    cost_function <- function(x){ 1 / ((1337.8 * abs(x[adj])^6) + (278.19 * abs(x[adj])^5) - (517.39 * abs(x[adj])^4) - (78.199 * abs(x[adj])^3) + (93.419 * abs(x[adj])^2) + (19.825 * abs(x[adj])) + 1.64) }
 
     #set the labels to be used within the returned plot
     main.title <- "Accumulated cost isolines around origin"
@@ -307,8 +306,8 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
   }
 
   if(funct=="wcs") {
-    #wheel critical slope cost function; tan(x*pi/180)*100 turns the slope from degrees to percent; the latter is requested by the cost function
-    cost_function <- function(x){ 1 / (1 + ((tan(x*pi/180)*100) / sl.crit)^2) }
+    #wheel critical slope cost function; the slope is requested as rise/run by the cost function
+    cost_function <- function(x){ 1 / (1 + ((abs(x[adj])*100) / sl.crit)^2) }
 
     #set the labels to be used within the returned plot
     main.title <- "Accumulated cost isolines around origin"
@@ -319,8 +318,8 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
 
   if(funct=="vl") {
     #Van Leusen's metabolic energy expenditure cost function
-    #note: V is velocity in m/s; tan(x*pi/180)*100 turns the slope from degrees to percent
-    cost_function <- function(x){ 1 / (1.5 * W + 2.0 * (W + L) * (L / W)^2 + N * (W + L) * (1.5 * V^2 + 0.35 * V * (tan(x*pi/180)*100) + 10)) }
+    #note: V is velocity in m/s; the slope is in percent
+    cost_function <- function(x){ 1 / (1.5 * W + 2.0 * (W + L) * (L / W)^2 + N * (W + L) * (1.5 * (V^2) + 0.35 * V * ((abs(x[adj])*100) + 10))) }
 
     #set the labels to be used within the returned plot
     main.title <- "Accumulated cost isolines around origin"
@@ -331,8 +330,8 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
 
   if(funct=="p") {
     #Pandolf et al.'s metabolic energy expenditure cost function
-    #note: V is velocity in m/s; tan(x*pi/180)*100 turns the slope from degrees to percent
-    cost_function <- function(x){ 1 / (1.5 * W + 2.0 * (W + L) * (L / W)^2 + N * (W + L) * (1.5 * V^2 + 0.35 * V * (tan(x*pi/180)*100))) }
+    #note: V is velocity in m/s; the slope is expressed in percent
+    cost_function <- function(x){ 1 / (1.5 * W + 2.0 * (W + L) * (L / W)^2 + N * (W + L) * (1.5 * (V^2) + 0.35 * V * (abs(x[adj])*100))) }
 
     #set the labels to be used within the returned plot
     main.title <- "Accumulated cost isolines around origin"
@@ -343,35 +342,45 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
 
   #cost calculation for walking-speed-based cost functions
   if (funct=="t" | funct=="tofp" | funct=="mt" | funct=="ic" | funct=="icofp") {
-    #calculate the walking speed for each cell of the slope raster
-    speed.kmh <- raster::calc(slope, cost_function)
+
+    #restrict the speed calculation to adjacent cells by creating an index for adjacent cells (adj) with the function 'adjacent'
+    adj <- raster::adjacent(dtm, cells=1:ncell(dtm), pairs=TRUE, directions=8)
+
+    speed <- slope
+
+    #apply the cost function to the adjacent cells of the speed dataset, which is equal to the slope dataset as per previous step
+    speed[adj] <- cost_function(slope)
 
     #turn the walking speed from kmh to ms (0.278=1000/3600)
-    speed.ms <- speed.kmh * 0.278
+    speed <- speed * 0.278
 
-    #create a transitional matrix for the speed in ms
-    cost.TRANS <- gdistance::transition(speed.ms, transitionFunction=mean, directions=moves)
+    #correct the speed values taking into account the distance between cell centers
+    Conductance <- gdistance::geoCorrection(speed)
   }
 
   #cost calculation for other types of cost functions;
   #note the Uriarte Gonzalez's slope-dependant walking-time cost function is in this group since (unlike the above functions)
   #it expresses cost as time NOT speed
   if (funct=="ree" | funct=="hrz" | funct=="wcs" | funct=="vl" | funct=="p" | funct=="ug") {
-    #calculate the cost for each cell of the slope raster
-    cost <- raster::calc(slope, cost_function)
 
-    #create a transitional matrix for the cost
-    cost.TRANS <- gdistance::transition(cost, transitionFunction=mean, directions=moves)
+    #restrict the cost calculation to adjacent cells by creating an index for adjacent cells (adj) with the function 'adjacent'
+    adj <- raster::adjacent(dtm, cells=1:ncell(dtm), pairs=TRUE, directions=8)
+
+    cost <- slope
+
+    #apply the cost function to the adjacent cells of the cost dataset, which is equal to the slope dataset as per previous step
+    cost[adj] <- cost_function(slope)
+
+    #correct the cost values taking into account the distance between cell centers
+    Conductance <- gdistance::geoCorrection(cost)
   }
 
-  #geocorrection of the cost matrix
-  Conductance <- gdistance::geoCorrection(cost.TRANS)
-
-  #accumulate the pace outwards from the origin
+  #accumulate the cost outwards from the origin
   accum_final <- gdistance::accCost(Conductance, sp::coordinates(origin))
 
-  #if user select the Tobler's, the modified Tobler's, or the Uriarte Gonzalez's function, turn seconds into the user-defined time-scale
-  if (funct=="t" | funct=="tofp" | funct=="mt" | funct=="ic" | funct=="ug") {
+  #if user select the Tobler's, the modified Tobler's, the Irmischer-Clarke's,
+  #or the Uriarte Gonzalez's functions, turn seconds into the user-defined time-scale
+  if (funct=="t" | funct=="tofp" | funct=="mt" | funct=="ic" | funct=="icofp" | funct=="ug") {
     if (time=="h") {
       #turn seconds into hours
       accum_final <- accum_final / 3600
@@ -392,6 +401,10 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
     breaks <- round((max(accum_final[][is.finite(accum_final[])]) - min(accum_final[][is.finite(accum_final[])])) / 10,2)
   }
 
+  #crop the final accumulated dataset to the extent of the input dtm so that NA cell (e.g., cells corresponding to the sea)
+  #can be excluded
+  accum_final <- raster::mask(accum_final, dtm)
+
   #set the break values for the isolines, again excluding inf values
   levels <- seq(min(accum_final[][is.finite(accum_final[])]), max(accum_final[][is.finite(accum_final[])]), breaks)
 
@@ -410,7 +423,7 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
          cex.main=0.95,
          cex.sub=0.75,
          legend.lab=legend.cost,
-         col = terrain.colors(255))
+         col = topo.colors(255))
     raster::contour(accum_final,
             add=TRUE,
             levels=levels,
@@ -499,6 +512,10 @@ movecost <- function (dtm, slope=NULL, origin, destin=NULL, funct="t", time="h",
     par(mfrow = c(1,1))
   }
 
+  #restore the advice for error messages on the R console, which has been deactivated
+  #at the beginning of function
+  options(warn = 1)
+
   results <- list("accumulated.cost.raster"=accum_final,
                   "isolines" = isolines,
                   "LCPs"=sPath,
